#========================================================================================
# package      DE_EPAGES::Customer::API::Object::CustomerGroup
# state        public
# base         DE_EPAGES::Permission::API::Object::Group
# description  object interface for CustomerGroup.
#========================================================================================
package DE_EPAGES::Customer::API::Object::CustomerGroup;
use base DE_EPAGES::Permission::API::Object::Group;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $CustomerGroup->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'CustomerGroup' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $CustomerGroup->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CustomerGroup = DE_EPAGES::Customer::API::Object::CustomerGroup->insert($hInfo);
# example      my $Class = LoadClassByAlias('CustomerGroup');
#               my $CustomerGroup = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertCustomerGroup' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $CustomerGroup | new object | object
# hook         OBJ_InsertCustomerGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomerGroupID - customer group id    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;


    my $Shop = $hInfo->{'Shop'} || $hInfo->{Parent}->getSite();

    # NewCustomer is a predefined identifier for price lists in Unity
    if ( $Shop->isUnity()) {
        $Shop->featureIncrementValue('CustomerGroups') unless ($hInfo->{'Alias'} eq 'NewCustomer');
    } else {
        $Shop->featureIncrementValue('CustomerGroups');
    }
    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertCustomerGroup', {'Object' => $self, 'CustomerGroupID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Shop ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CustomerGroup->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteCustomerGroup' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteCustomerGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomerGroupID - customer group id    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    # decrement the usage counter of the CustomerGroups feature
    my $Shop = $self->getSite() ;
    TriggerHook('OBJ_DeleteCustomerGroup', {'Object' => $self, 'CustomerGroupID' => $self->id });
    $_->set({'CustomerGroup'=>undef}) foreach @{$self->get('Customers')};
    $self->SUPER::delete;
    $Shop->featureIncrementValue('CustomerGroups', -1);

    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CustomerGroup->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateCustomerGroup' and 'OBJ_AfterUpdateCustomerGroup'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateCustomerGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomerGroupID - customer group id    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateCustomerGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomerGroupID - customer group id    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateCustomerGroup', {'Object'=>$self, 'Values'=> $hValues, 'CustomerGroupID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateCustomerGroup',  {'Object'=>$self, 'Values'=> $hValues, 'CustomerGroupID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Customer::API::Object::CustomerGroup->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Shop' => $Parent->getSite(),
    };
}

1;
