#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::Payments
# state        public
# description  Helper module for generating MolliePayments Payments requests to epagesj
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::Payments;
use base DE_EPAGES::MolliePayments::API::Requests::BaseRequest;

use DE_EPAGES::MolliePayments::API::Tools qw ( FormatFloatForMollie );

use constant POST_HTTP_METHOD => 'POST';
use constant GET_HTTP_METHOD => 'GET';
use constant PUT_HTTP_METHOD => 'PUT';

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::Requests::Payments->new($hMethodsData);
#----------------------------------------------------------------------------------------
# description  MolliePayments Payments requests to epagesj helper constructor
#----------------------------------------------------------------------------------------
# input        $hMethodsData | required data to generate Payments requests to epagesj | hash.ref
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Payments
#========================================================================================
sub new {
    my $class = shift;
    my ($hData) = @_;

    my $self = $class->SUPER::new($hData);
    $self->{'urlPath'} = {};

    return $self;
}

#========================================================================================
# function     withPaymentId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withPaymentId($PaymentId);
#----------------------------------------------------------------------------------------
# description  Set payment id as part of the payload
#----------------------------------------------------------------------------------------
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Payments
#========================================================================================
sub withPaymentId {
    my $self = shift;
    my ($PaymentId) = @_;

    $self->{'urlPath'}->{'paymentId'} = $PaymentId;

    return $self;
}


#========================================================================================
# function     withRefundAmountAndCurrencyId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withRefundAmountAndCurrencyId($Amount, $CurrencyId);
#----------------------------------------------------------------------------------------
# description  Set Amount as part of the payload
#----------------------------------------------------------------------------------------
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Payments
#========================================================================================
sub withRefundAmountAndCurrencyId {
    my $self = shift;
    my ($Amount, $CurrencyId) = @_;

    $self->{'payload'}->{'amount'} = {
        'value' => FormatFloatForMollie($Amount),
        'currency' => $CurrencyId
    };

    return $self;
}

#========================================================================================
# function     getEpagesjRequestSuffixUrlPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = $MethodsRequest->getEpagesjRequestSuffixUrlPath( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the Payments postfix url path from the epagesj endpoint
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $Url | Payments suffix url path from the epagesj endpoint | string
#========================================================================================
sub getEpagesjRequestSuffixUrlPath {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    my $urlPath =  '/payments/' . $self->{'urlPath'}->{'paymentId'};

    if ($HttpRequestMethod eq POST_HTTP_METHOD) {
        return $urlPath .= '/refunds';
    }

    if ($HttpRequestMethod eq GET_HTTP_METHOD || $HttpRequestMethod eq PUT_HTTP_METHOD) {
        return $urlPath;
    }

}

#========================================================================================
# function     getHttpHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestHeader = $self->getHttpHeaders( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns required headers of the Payments request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hRequestHeader | Payments request headers hash | hash.ref
#========================================================================================
sub getHttpHeaders {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq POST_HTTP_METHOD ||
        $HttpRequestMethod eq GET_HTTP_METHOD ||
        $HttpRequestMethod eq PUT_HTTP_METHOD) {
        return {
            'X-AccessToken' => $self->{'data'}->{'payment'}->get('AccessToken')
        };
    }

}

#========================================================================================
# function     getPayload
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPayload = $self->getPayload( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the body payload of the Payments request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hPayload | Payments request payload hash | hash.ref
#========================================================================================
sub getPayload {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq POST_HTTP_METHOD) {
        return $self->{'payload'};
    }

    if ($HttpRequestMethod eq PUT_HTTP_METHOD) {
        my $BasketOrder = $self->{'data'}->{'container'}->parent();
        $self->{'payload'}->{'description'} = $BasketOrder->alias();

        return $self->{'payload'};
    }

}

#========================================================================================
# function     getQueryParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $QueryParams = $self->getQueryParams();
#----------------------------------------------------------------------------------------
# description  It returns the query Params of the Payments request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $QueryParams | Payments request $QueryParams string | string
#========================================================================================
sub getQueryParams {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq GET_HTTP_METHOD) {
        return $self->{'queryParams'};
    }
}

#========================================================================================
# function     handleResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFilteredResponse = $self->handleResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  It returns the response filtered
#----------------------------------------------------------------------------------------
# input        $hResponse | response hash | hash.ref
# return       $hFilteredResponse | filtered response hash | hash.ref
#========================================================================================
sub handleResponse {
    my $self = shift;
    my ($hResponse) = @_;

    return $hResponse;
}

1;
