use strict;
use warnings;
use Getopt::Long;
use Time::HiRes qw( time );

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::MolliePayments::API::Cartridge;
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         InitializeMolliePaymentsDashboardNotification
# state            public
#----------------------------------------------------------------------------------------
# syntax           InitializeMolliePaymentsDashboardNotification();
#----------------------------------------------------------------------------------------
# description      Adds and activates the notification for the MolliePayments
#                       to all Unity shops that have the related feature enabled.
#========================================================================================
sub InitializeMolliePaymentsDashboardNotification {
    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::MolliePayments::API::Cartridge->new;

    foreach my $ShopID ( @$aShopIDs ) {
        my $Shop = LoadObject($ShopID);
        next unless $Shop->featureMaxValue('MolliePayments');
        if(isPaymentMethodMollieInstalledAndActivated($Shop))
        {
            print "PaymentMethod 'Mollie' is activated for ShopID: ". $ShopID.", Shop path:  ".$Shop->pathString." \n " ;
            eval {
                # import shop notifications
                ImportObjects($Cartridge->xmlDirectory() . '/ShopNotificationMolliePayments.xml', $Shop->pathString)
                    unless $Shop->folder('ShopNotifications')->existsChild('MolliePayments');
            };
            warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".") if ExistsError();
            print "Notification imported. ShopID: ". $ShopID.", Shop path:  ".$Shop->pathString." \n " ;
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub isPaymentMethodMollieInstalledAndActivated {
    my ($Shop) = @_;

    my $PaymentMethods = $Shop->get('PaymentMethods');
    foreach my $PaymentMethod (@$PaymentMethods){
        if ($PaymentMethod->class->alias eq 'PaymentMethodMollie') {
            if ( $PaymentMethod->get('IsActivated') ) {
                return 1;
            }
        }
    }
    return 0;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeMolliePaymentsDashboardNotification(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds and activates the notification for the MolliePayments
    to all existing Unity shops.
Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);
