#========================================================================================
# package      DE_EPAGES::MolliePayments::UI::Shop
# base         DE_EPAGES::Order::UI::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  UI Controller for shop
#========================================================================================
package DE_EPAGES::MolliePayments::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::MolliePayments::API::PaymentStatusService;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Presentation::API::DereferrerToken qw ( ValidateDereferrerToken );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID );
use DE_EPAGES::MolliePayments::API::Constants qw (
    INTERNAL_CARTRIDGE_NAME
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    PAYMENT_STATUS_PAID
);

use strict;



#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  Name of PaymentType for search/view transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {
    return INTERNAL_CARTRIDGE_NAME;
}



#========================================================================================
# function     MolliePaymentsPaymentWebhook
# state        public
#----------------------------------------------------------------------------------------
# syntax       MolliePaymentsPaymentWebhook( $Servlet );
#----------------------------------------------------------------------------------------
# description  Handles LineItemPayment status updating asynchronously.
#               As stated in Mollie documentation (https://docs.mollie.com/guides/webhooks)
#               payment webhook gets only triggered at "paid", "expired", "failed" and
#               "canceled" status changes.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub MolliePaymentsPaymentWebhook {
    my $self = shift;
    my ($Servlet) = @_;

    $self->_LogPayment('Received MolliePaymentsPaymentWebhook',{
        'DereferrerToken' => $Servlet->params('DereferrerToken'),
        'LineItemPaymentGUID' => $Servlet->params('LineItemPaymentGUID'),
        'PaymentId' => $Servlet->params('id'),
    });

    my $PaymentStatusService = $self->_createPaymentStatusService();

    my $LineItemPayment;
    my $NewPaymentStatus;
    my $BasketOrOrder;

    eval {
        $self->_authenticateCallback($Servlet);

        $LineItemPayment = LoadObjectByGUID($Servlet->params('LineItemPaymentGUID'));
        $NewPaymentStatus = $PaymentStatusService->handleStatusOnPaymentWebhook(
            $LineItemPayment,
            $Servlet->params('id')
        );

        $BasketOrOrder = $LineItemPayment->container()->parent();
    };

    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment('Error on MolliePaymentsPaymentWebhook', {'Error' => $Error->message()});

    } elsif ( ($NewPaymentStatus eq PAYMENT_STATUS_PAID) && $BasketOrOrder->instanceOf('Basket') ) {

            my $Order = DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $BasketOrOrder);

            $PaymentStatusService->setOrderStatusPaidTime($Order, $LineItemPayment->get('TransTime'));

            $self->_LogPayment('Created Order for Basket via Webhook', {
                'BasketId' => $BasketOrOrder->id(),
                'OrderId' => $Order->id(),
                'PaidOn' => $Order->get('PaidOn')
            });
    }

    $Servlet->params('ContentType', 'text/plain');
    $Servlet->writeContentOutput('OK');

    return;
}

#========================================================================================
sub _authenticateCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $ObjectId = $Servlet->params('ObjectID');
    my $LineItemPaymentId = $Servlet->params('LineItemPaymentGUID');

    my $PayLoad = sprintf('%s&%s&%s', $Shop->alias(), $ObjectId, $LineItemPaymentId);
    my $Token = $Servlet->params('DereferrerToken');

    $self->_ValidateDereferrerToken($Token, $PayLoad);

    return;
}
#========================================================================================
sub _ValidateDereferrerToken {
    my $self = shift;
    my ($Token, $URL) = @_;
    ValidateDereferrerToken($Token, $URL);
}
#========================================================================================
sub _createPaymentStatusService {
    my $self = shift;
    return DE_EPAGES::MolliePayments::API::PaymentStatusService->new();
}
#========================================================================================
sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}
1;