#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::Requests_Payments
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::Requests::Payments
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::Requests_Payments;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::MolliePayments::API::Tools qw ( FormatFloatForMollie );

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify::Sut;
use Test::More;

use strict;

use constant ACCESS_TOKEN => 'MyAccessToken';
use constant CUSTOMER_ORDER_ID => 'MyOrderId';
use constant MOLLIE_PAYMENT_ID => 'tr_DPbpxrjRqa';
use constant PAYMENTS_SUFFIX_EPAGESJ_PATH => '/payments/' . MOLLIE_PAYMENT_ID;
use constant REFUND_QUANTITY => '24.95';
use constant REFUND_CURRENCY => 'EUR';
use constant REFUND_AMOUNT => { 'value' => FormatFloatForMollie(REFUND_QUANTITY), 'currency' => REFUND_CURRENCY };

sub test {
    my $self = shift;

    $self->test_Payment_Request_creation();
    $self->test_Payment_provision_for_get_requests();
    $self->test_Payment_provision_for_post_requests();
    $self->test_Payment_provision_for_put_requests();

    done_testing();
}

sub test_Payment_Request_creation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    is( scalar(keys %{$SUT->{'data'}}), 2, "$SubTestName - Payments object gets initialized with some arguments.");

    is( scalar(keys %{$SUT->{'payload'}}), 0, "$SubTestName - Payments object gets initialized with an empty payload.");
}

sub test_Payment_provision_for_get_requests {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    $SUT->withPaymentId(MOLLIE_PAYMENT_ID);

    is(
        $SUT->getEpagesjRequestSuffixUrlPath('GET'),
        PAYMENTS_SUFFIX_EPAGESJ_PATH,
        "$SubTestName - Payments object provides epagesj suffix path for GET requests."
    );
    is_deeply(
        $SUT->getQueryParams('GET'),
        {},
        "$SubTestName - Payments object provides empty hash query parameters for GET requests."
    );
    is_deeply(
        $SUT->getHttpHeaders('GET'),
        { 'X-AccessToken' => ACCESS_TOKEN },
        "$SubTestName - Payments object provides HTTP headers for GET requests."
    );
}

sub test_Payment_provision_for_post_requests {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    $SUT->withPaymentId(MOLLIE_PAYMENT_ID);
    $SUT->withRefundAmountAndCurrencyId(REFUND_QUANTITY, REFUND_CURRENCY);

    is(
        $SUT->getEpagesjRequestSuffixUrlPath('POST'),
        PAYMENTS_SUFFIX_EPAGESJ_PATH . '/refunds',
        "$SubTestName - Payments object provides epagesj suffix path for POST requests."
    );
    is_deeply(
        $SUT->getHttpHeaders('POST'),
        { 'X-AccessToken' => ACCESS_TOKEN },
        "$SubTestName - Payments object provides HTTP headers for POST requests."
    );
    is_deeply(
        $SUT->getPayload('POST'),
        { 'amount' => REFUND_AMOUNT },
        "$SubTestName - Payments object provides payload for POST requests."
    );
}

sub test_Payment_provision_for_put_requests {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    $SUT->withPaymentId(MOLLIE_PAYMENT_ID);

    is(
        $SUT->getEpagesjRequestSuffixUrlPath('PUT'),
        PAYMENTS_SUFFIX_EPAGESJ_PATH,
        "$SubTestName - Payments object provides epagesj suffix path for PUT requests."
    );
    is_deeply(
        $SUT->getHttpHeaders('PUT'),
        { 'X-AccessToken' => ACCESS_TOKEN },
        "$SubTestName - Payments object provides HTTP headers for PUT requests."
    );
    is_deeply(
        $SUT->getPayload('PUT'),
        { 'description' => CUSTOMER_ORDER_ID },
        "$SubTestName - Payments object provides payload for PUT requests."
    );
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $hData = $self->_generateRequestData();
    my $aParameterList = [$hData];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::API::Requests::Payments', $aParameterList);

    return $MockifySut->getMockObject();
}

sub _generateRequestData {
    my $self = shift;

    return {
        'payment' => $self->_createPaymentMethod(),
        'container' => $self->_createLineItemContainer()
    };
}

sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'AccessToken' => ACCESS_TOKEN
    };
    my $MockifiyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifiyPaymentMethod->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );
    $MockifyContainer->addMockWithReturnValue('parent', $self->_createCustomerOrder());

    return $MockifyContainer->getMockObject();
}

sub _createCustomerOrder {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );
    $MockifyOrder->addMockWithReturnValue('alias', CUSTOMER_ORDER_ID);

    return $MockifyOrder->getMockObject();
}

#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();