use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadObjectByPath );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         PatchUnityProductVideoProvider
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchUnityProductVideoProvider();
#----------------------------------------------------------------------------------------
# description      Moves content from attribute YoutubeVideoId to attribute
#                   ProductVideoProvider as JSON string
#========================================================================================
sub PatchUnityProductVideoProvider {
    my ($ShopAlias) = @_;

    # test if attribute YoutubeVideoId still exists
    my $ProductClass = LoadClassByAlias('Product');

    unless ($ProductClass->existsAttribute('YoutubeVideoId')){
        print "Nothing to patch. Attribute YoutubeVideoId not exists.\n";
        return;
    }
    my $YoutubeVideoAttributeID = $ProductClass->attribute('YoutubeVideoId')->id;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            # Unity shop
            _PatchUnityProductVideoProvider($Shop, $YoutubeVideoAttributeID);
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                _PatchUnityProductVideoProvider($Shop, $YoutubeVideoAttributeID);
            };
            warn('Patch of the ProductVideoProvider failed for shop ID ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub _PatchUnityProductVideoProvider {
    my ($Shop,$YoutubeVideoAttributeID) = @_;

    #select all products of this shop with youtube video
    my $dbi= GetCurrentDBHandle();

    #get closed now shops where sf is not already closed
    my @Parameter = ($Shop->id, $YoutubeVideoAttributeID);

    my $dbResult = $dbi->execute(<<END_SQL, \@Parameter, ['Integer','Integer','String'], [qw(ProductID LanguageID Value)]);
SELECT objectid, languageid, value FROM localizedstringattribute
INNER JOIN product ON productid=objectid
WHERE shopid=? and attributeid=?;
END_SQL
    foreach my $hEntry (@$dbResult) {
        my $Product = LoadObject($hEntry->{'ProductID'});
        my $ProductVideoProvider = $Product->get('ProductVideoProvider', $hEntry->{'LanguageID'});
        #only set unset attributes
        next if defined  $ProductVideoProvider;
        my $VideoJSON = '[{"VideoType":"YouTube","VideoID":"' . $hEntry->{'Value'} . '"}]';
        $Product->set({'ProductVideoProvider' => $VideoJSON }, $hEntry->{'LanguageID'});
    }
    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopid=s'    => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchUnityProductVideoProvider($ShopAlias); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Moves content from attribute YoutubeVideoId to attribute PatchUnityProductVideoProvider as JSON string
Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );