#========================================================================================
# package      DE_EPAGES::ProductRecommendation::UI::Product
# base         DE_EPAGES::Presentation::UI::Object
# state        private
#========================================================================================
package DE_EPAGES::ProductRecommendation::UI::Product;
use base qw ( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Html qw ( RemoveHtmlTags );
use DE_EPAGES::Core::API::Log qw( LogError GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Customer::API::Shop qw ( GetCustomerForUser );
use DE_EPAGES::ProductRecommendation::API::Inquiry qw( SendProductInquiryMail );
use DE_EPAGES::ProductRecommendation::API::Recommendation qw( SendProductRecommendationMail );
use DE_EPAGES::ProductRecommendation::API::Table::ProductRecommendationUser qw(
    InsertProductRecommendationUser
    ExistsProductRecommendationUser
    GetInfoProductRecommendationUser
    UpdateProductRecommendationUser
);

sub SendRecommendation {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object();
    my $User = $Servlet->user;
    my $Form = $Servlet->form;
    my $hForm = $Form->form( $Product, 'ProductRecommendation' );

    $self->GoogleCaptcha($Servlet);

    #spam check - check if field senderName has value,
    #this field is only for spam protection it is hidden for the customer
    my $SpamCheck = $hForm->{'SenderName'};
    if($SpamCheck){
        $Form->executeFormError({
            'Name'   => 'Copy',
            'Form'   => 'ProductRecommendation',
            'Reason' => 'SentMessage',
        });
        return;
    }

    # do some tests if we can send ProductRecommendations
    _TestRequirements( $Servlet );

    # remove all html tags but show newlines, EPG-5734,EPG-9829,EPG-9286
    my $Greeting = $hForm->{'Greeting'};
    if( defined $Greeting ) {
        $Greeting =~ s/\r?\n/<br \/>/g;
        $Greeting = RemoveHtmlTags($Greeting);
        $Greeting =~ s/\r?\n/<br \/>/g;
    }

    # allow copy only for registered users
    my $Copy = ($hForm->{'Copy'} && defined $User && !$User->get('IsAnonymous')) ? 1 : 0;

    eval {
        SendProductRecommendationMail( $Product, {
            'Receiver'          => $hForm->{'Receiver'},
            'ReceiverEmail'     => $hForm->{'ReceiptEmail'},
            'Greeting'          => $Greeting,
            'Sender'            => $hForm->{'Sender'},
            'SenderEmail'       => $hForm->{'Email'},
            'CurrencyID'        => $Servlet->currencyID,
            'LocaleID'          => $Servlet->locale,
            'Copy'              => $Copy,
         });
    };
    if( ExistsError() ) {
        my $Error = GetError();
        GetLog->debug($Error->message);
        $Form->executeFormError({
            'Form'   => 'ProductRecommendation',
            'Name'   => 'To',
            'Reason' => 'SendMailFailed',
        });
    }

    if( $Copy ) {
        $Form->executeFormError({
            'Reason' => 'SentMessageWithCopy',
            'Form' => 'ProductRecommendation',
            'Name' => 'EmailNoError',
        });
    }
    else {
        $Form->executeFormError({
            'Reason' => 'SentMessage',
            'Form' => 'ProductRecommendation',
            'Name' => 'Copy',
        });
    }

    return;
}

sub SendInquiry {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    my $Form    = $Servlet->form;
    my $hForm   = $Form->form( $Product, 'ProductInquiry' );

    $self->GoogleCaptcha($Servlet);

    #spam check - check if field senderName has value,
    #this field is only for spam protection it is hidden for the customer
    my $SpamCheck = $hForm->{'SenderName'};
    if($SpamCheck){
        $Form->executeFormError({
            'Name'   => 'Copy',
            'Form'   => 'ProductInquiry',
            'Reason' => 'SentMessage',
        });
        return;
    }

    my $User = $Servlet->user;
    my $Copy = ($hForm->{'Copy'} && defined $User) ? 1 : 0;

    $Form->executeFormError({
        'Form'   => 'ProductInquiry',
        'Name'   => 'Permission',
        'Reason' => 'NotAllowed',
    }) unless $Shop->get('AllowInquiries');

    eval {
        SendProductInquiryMail( $Product, {
            'Sender'            => $hForm->{'Sender'},
            'SenderEmail'       => $hForm->{'Email'},
            'Inquiry'           => $hForm->{'Inquiry'},
            'Phone'             => $hForm->{'Phone'},
            'CurrencyID'        => $Servlet->currencyID,
            'LocaleID'          => $Servlet->locale,
            'Copy'              => $Copy,
        });
    };
    if( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
        $Form->executeFormError({
            'Form'   => 'ProductInquiry',
            'Name'   => 'To',
            'Reason' => 'SendMailFailed',
        });
    }

    if( $Copy ) {
        $Form->executeFormError({
            'Reason' => 'SentMessageWithCopy',
            'Form' => 'ProductInquiry',
            'Name' => 'EmailNoError',
        });
    }
    else {
        $Form->executeFormError({
            'Reason' => 'SentMessage',
            'Form' => 'ProductInquiry',
            'Name' => 'Copy',
        });
    }

    return;
}

sub _TestRequirements {
    my ( $Servlet ) = @_;

    my $Product = $Servlet->object();
    my $Shop = $Product->getSite();
    my $User = $Servlet->user;
    my $Form = $Servlet->form;


    # Recommendations switched off...
    if (not $Shop->get('AllowRecommendations')) {
        $Form->executeFormError({
            'Form'   => 'ProductRecommendation',
            'Name'   => 'Permission',
            'Reason' => 'NotAllowed',
        });
    }

    # allow Recommendation without counting for people wich are not a User
    # allow Recommendation without counting for anonymous users
    # allow Recommendation without counting for registered users
    return if $Shop->get('AnonymousRecommendation');

    # no Recommendation for anonymous or not defined Users
    $Form->executeFormError({
        'Form'   => 'ProductRecommendation',
        'Name'   => 'Permission',
        'Reason' => 'NotAllowedForAnonymous',
    }) if not defined $User or $User->get('IsAnonymous');


    # we have a registered user, so we have to count all Recommendations
    my $UserID = $User->id;
    my $ShopID = $Shop->id;

    if (ExistsProductRecommendationUser($UserID, $ShopID)) {
        my $hProductRecommendationUser = GetInfoProductRecommendationUser($UserID, $ShopID);
        my $LastDate = $hProductRecommendationUser->{'LastDate'};
        my $CurrentDate = _CurrentDate();
        my $cmp = DateTime->compare( $CurrentDate, $LastDate );
        my $DayCount = $hProductRecommendationUser->{'Number'};

        # is today?
        if ($cmp == 0) {
            # test limit
            if ($DayCount >= $Shop->get('MaxRecommendations') ) {
                $Form->executeFormError({
                    'Form'   => 'ProductRecommendation',
                    'Name'   => 'Permission',
                    'Reason' => 'RecommendationLimitExceeded',
                });
            }
            # increment counter for the day
            $DayCount ++;
        }
        # update RecommendationUser
        $hProductRecommendationUser = {
            'ShopID' => $ShopID,
            'UserID' => $UserID,
            'Number' => 0 == $cmp ? $DayCount: 1 ,
            'LastDate' => $CurrentDate,
        };
        UpdateProductRecommendationUser($hProductRecommendationUser);

    } else {
        # insert new RecommendationUser
        my $LastDate = _CurrentDate();
        my $hProductRecommendationUser = {
            'ShopID' => $ShopID,
            'UserID' => $UserID,
            'Number' => 1,
            'LastDate' => $LastDate,
        };
        InsertProductRecommendationUser($hProductRecommendationUser);
    }

    return;

}

sub ViewRecommendation {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object();
    my $Shop = $Product->getSite();
    my $ShopID = $Shop->id;
    my $User = $Servlet->user;

    my $Counter;
    my $NewDay;
    if (defined $User) {
        my $UserID = $User->id;
        if (ExistsProductRecommendationUser($UserID, $ShopID)) {
            my $hProductRecommendationUser = GetInfoProductRecommendationUser($UserID, $ShopID);
            # Set the TLE-variable Counter
            $Counter = $hProductRecommendationUser->{'Number'};

            # Set the TLE-Variable NewDay
            my $LastDate = $hProductRecommendationUser->{'LastDate'};
            my $CurrentDate = _CurrentDate();
            my $cmp = DateTime->compare( $CurrentDate, $LastDate );
            if ( 0 == $cmp ) {
                $NewDay = 0;
            } else {
                $NewDay = 1;
            }
        }
        else {
            $Counter = 0;
            $NewDay = 0;
        }

        # CustomerEMail
        my $Customer = GetCustomerForUser($User, $Shop);
        my $BillingAddress = defined $Customer ? $Customer->get('BillingAddress') : undef;
        my $EMail =  $BillingAddress ? $BillingAddress->get('EMail') : '';
        $Servlet->vars('CustomerEMail',$EMail);

        # Customername
        my $Name;
        if( $BillingAddress ) {
            if( $BillingAddress->get('DisplayName') ) {
                $Name = $BillingAddress->get('DisplayName')
            }
            else {
                $Name = $BillingAddress->get('FullName');
            }
        }
        $Servlet->vars('CustomerName',$Name);
    }
    else {
        $Counter = 0;
        $NewDay = 0;
        $Servlet->vars('CustomerEMail','');
    }
    $Servlet->vars( 'Counter' => $Counter );
    $Servlet->vars( 'NewDay' => $NewDay );
}

sub ViewInquiry {
    my $self = shift;
    my ($Servlet) = @_;

    my $User    = $Servlet->user;
    my $hInfo;
    my $Name = '';

    if (defined $User) {
        my $BillingAddress = $User->get('BillingAddress');
        $hInfo = $BillingAddress->get([qw( EMail Phone DisplayName FullName )]) if $BillingAddress;
        if ($hInfo->{'DisplayName'}) {
            $Name = $hInfo->{'DisplayName'};
        } else {
            $Name = $hInfo->{'FullName'};
        }
    }
    $Servlet->vars('CustomerEMail', $hInfo->{'EMail'} ? $hInfo->{'EMail'} : '');
    $Servlet->vars('CustomerPhone', $hInfo->{'Phone'} ? $hInfo->{'Phone'} : '');
    $Servlet->vars('CustomerName',  $Name);
}

sub _CurrentDate {
    my $CurrentDate = GetCurrentDateTime();

    $CurrentDate->set(
        hour        => 0,
        minute      => 0,
        second      => 0,
        nanosecond  => 0,
    );

    return $CurrentDate;
}
1;
