use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::File qw ( CopyFile fsplit );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath );
use DE_EPAGES::Shop::API::Cartridge;
use DE_EPAGES::Shop::API::Constants qw ( UNITY_ORDERDOCUMENTS_LOGO );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         PatchOrderDocumentsLogo
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchOrderDocumentsLogo();
#----------------------------------------------------------------------------------------
# description      Copies the current shop logo as OrderDocuments logo, set the logo to the shop attribute
#                   OrderDocumentsLogo and activates the attribute UseUnityShopLogo
#========================================================================================
sub PatchOrderDocumentsLogo {
    my ($ShopAlias) = @_;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            # Unity shop
            _copyOrderDocumentsLogo($Shop);           
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        print "Reading all Unity shops\n";
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                _copyOrderDocumentsLogo($Shop);
            };
            warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub _copyOrderDocumentsLogo {
    my ($Shop) = @_;

    my $ShopAlias = $Shop->get('Alias');


    # attribute InitialShopWizardStartup is set, we change nothing....
    if($Shop->get('InitialShopWizardStartup')){
        print 'ShopWizard not finished yet. No action for shop: ' . $ShopAlias . ".\n";
        return;
    }

    # attribute is already set, nothing to do....
    if($Shop->get('UseUnityShopLogo')){
        print 'UseUnityShopLogo is already set for shop ' . $ShopAlias . ".\n";
        return;
    }

    my $ShopLogo = $Shop->get('Logo');
    if (not $ShopLogo) {
        print 'No shop logo exists for shop (only attribute UseUnityShopLogo is set to 1 ) ' . $ShopAlias . ".\n";
    } else {
        # copy the shop logo as order documents logo...
        my $SourceFilePathAndName = $Shop->get('PublicPath') . '/'. $ShopLogo;
        my (undef, undef, $Extension) = fsplit($SourceFilePathAndName);
        my $TargetFileName = UNITY_ORDERDOCUMENTS_LOGO . $Extension;
        my $TargetFilePathAndName = $Shop->get('PublicPath') . '/'. $TargetFileName ;
        CopyFile($SourceFilePathAndName, $TargetFilePathAndName, {create_directory => 1});
        $Shop->set({'OrderDocumentsLogo' => $TargetFileName });
    }
    
    $Shop->set({'UseUnityShopLogo' => 1});

    return;

}


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopalias=s' => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchOrderDocumentsLogo($ShopAlias); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Copies the current shop logo as OrderDocuments logo, set the logo to the shop attribute
    OrderDocumentsLogo and activates the attribute UseUnityShopLogo

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );