#==============================================================================
# package      DE_EPAGES::ShopMailType::API::TLE::MailTypeTemplateHandler
# state        public
#==============================================================================
package DE_EPAGES::ShopMailType::API::TLE::MailTypeTemplateHandler;

use strict;
use DE_EPAGES::Core::API::File qw ( ExistsFile fsplit );
use DE_EPAGES::Core::API::Image qw ( GetImageInfo ResizeImage );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::TestParameter qw( TestObject );
use DE_EPAGES::Shop::API::Constants qw ( UNITY_MAIL_LOGO );

sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'MailTypeTemplateHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
}

sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'MailTypeTemplateHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'MAILTYPETEMPLATES');
    $Processor->registerHandler('FunctionHandler', $self, 'MAILTYPETEMPLATESADMIN');
    $Processor->registerHandler('FunctionHandler', $self, 'GetMailLogo');

    return;
}

#========================================================================================
# function     MAILTYPETEMPLATES
# state        public
#----------------------------------------------------------------------------------------
# syntax       #MAILTYPETEMPLATES(#Object)
# example      #LOOP(#MAILTYPETEMPLATES(#Object))
#                 #NameOrAlias<br/>
#               #ENDLOOP
#----------------------------------------------------------------------------------------
# description  Returns all shop mailtypetemplate objects that belongs to the group and
#               is sent to the user.
#----------------------------------------------------------------------------------------
# input        #Object | mailtypetemplategroup object | object
#========================================================================================
sub MAILTYPETEMPLATES { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($MailTypeTemplateGroup) = @$aParams;

    return [
        map { $_->tleHash }
        grep { $_->get('HasToField') == 0 }
        @{$MailTypeTemplateGroup->getMailTypeTemplates}
    ];
}
#========================================================================================
# function     MAILTYPETEMPLATESADMIN
# state        public
#----------------------------------------------------------------------------------------
# syntax       #MAILTYPETEMPLATESADMIN
# example      #LOOP(#MAILTYPETEMPLATESADMIN(#Shop))
#                 #NameOrAlias<br/>
#               #ENDLOOP
#----------------------------------------------------------------------------------------
# description  Returns all shop mailtypetemplate objects that is sent to the admin.
#========================================================================================
sub MAILTYPETEMPLATESADMIN { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;

    return [
        map { $_->tleHash }
        grep { $_->get('HasToField') == 1 }
        @{LoadRootObject()->child('ShopMailTypeTemplates')->children}
    ];
}
#========================================================================================
# function     GetMailLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetMailLogo(#LogoPath, #MaxWidth, #MaxHeight)
#----------------------------------------------------------------------------------------
# description  Resizes the shop logo to fit into the given dimensions and returns the
#               path of the resized logo.
#----------------------------------------------------------------------------------------
# input        #Shop | shop object | object
# input        #MaxWidth | max width in px | integer
# input        #MaxHeight | max height in px | integer
# return       #MailLogoPath | web path of the logo for mails | string
#========================================================================================
sub GetMailLogo {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $MaxWidth, $MaxHeight) = @$aParams;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $hInfoShop = $Shop->get(['OrderDocumentsLogo', 'PublicPath', 'WebPath']);

    return unless defined $hInfoShop->{'OrderDocumentsLogo'};

    # construct logo file name
    my $LogoPath = $hInfoShop->{'PublicPath'} . '/' . $hInfoShop->{'OrderDocumentsLogo'};

    return unless ExistsFile($LogoPath);

    my (undef, undef, $FileExtension ) = fsplit($hInfoShop->{'OrderDocumentsLogo'});

    my $MailLogoFileName = UNITY_MAIL_LOGO . $FileExtension;
    my $MailLogoPath = $hInfoShop->{'PublicPath'} . '/' . $MailLogoFileName;

    my $IsMailLogoAlreadyPresent = ExistsFile($MailLogoPath);
    # create resized logo in case the shop logo changed or we do not have a resized logo yet
    if (($IsMailLogoAlreadyPresent
            && GetImageInfo($LogoPath)->{'LastChange'} > GetImageInfo($MailLogoPath)->{'LastChange'})
        || !$IsMailLogoAlreadyPresent) {
        # resize image
        ResizeImage( $LogoPath, $MailLogoPath, $MaxWidth, $MaxHeight, 100 );
    }

    # return (relative) image path
    return $hInfoShop->{'WebPath'} . '/' . $MailLogoFileName;
}

1;