#==============================================================================
# $package      DE_EPAGES::ShopWizard::UI::ShopWizard
# $state        private
#==============================================================================
package DE_EPAGES::ShopWizard::UI::ShopWizard;
use base qw ( DE_EPAGES::Presentation::UI::Object );

use JSON;

use strict;
use DE_EPAGES::Core::API::Error                         qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File                          qw ( ExistsFile DeleteFile );
use DE_EPAGES::Core::API::Image                         qw ( ScaleImage );
use DE_EPAGES::Core::API::Log                           qw ( LogError );
use DE_EPAGES::Core::API::TestParameter                 qw ( TestInteger );
use DE_EPAGES::Design::API::Style                       qw ( DeleteTemporaryStyles );
use DE_EPAGES::Object::API::Factory                     qw ( LoadRootObject LoadObjectByPath );
use DE_EPAGES::Presentation::API::Session               qw ( GetSessionAttribute SetSessionAttribute );
use DE_EPAGES::ShopWizard::API::DemoContent             qw ( AddEBizElementsToNavBar ProcessEbizContent );
use DE_EPAGES::ShopWizard::API::ShopWizard              qw ( SaveWizard ChangeDesignStyle );
use DE_EPAGES::Trigger::API::Trigger                    qw ( TriggerHook );
use DE_EPAGES::ShopWizard::API::Constants               qw ( $WIZARD_DEFAULT_COLORS );

#==============================================================================
# Event handlers
#==============================================================================
sub SaveWizardAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    # EPG-10403: there is a special JavaScript monitor for this page
    $Servlet->monitor->blockTimeOut;

    my $Shop = $Servlet->object();

    # merge stored data from session with supplied form data
    my $hShopWizardAttributes = {
        # We need the values utf8-encoded to store them, so do NOT utf8-decode the json string.
        %{JSON->new->utf8(0)->decode( GetSessionAttribute( $Servlet->intSessionID, 'ShopWizardData') )},
        %{$Servlet->form->form($Shop, 'ShopWizardFinish')},
    };

    TriggerHook('UI_BeforeShopWizard', {
        'self'    => $self,
        'Servlet' => $Servlet,
        'Data'    => $hShopWizardAttributes,
        'Shop'    => $Shop
    });

    # set the selected branch as an shop attribute
    my $SelectedBranch;
    my $SessionAttribute = GetSessionAttribute( $Servlet->intSessionID, 'ShopWizardSelectedBranch');
    if($SessionAttribute) {
        $SelectedBranch = $Shop->loadObject(from_json($SessionAttribute)->{'ID'}, 'StyleGroup');
    } else {
        $SelectedBranch = LoadObjectByPath('/StyleGroups/Branches/StyleGroups/Others');
    }
    $Shop->set( { 'SelectedBranch' => $SelectedBranch } );

    #set design style as default an not temporary, then clean styles by deleting the temporaries
    my $StyleTemplateID = $Servlet->params('StyleTemplate');
    my $StyleTemplate;
    $StyleTemplate = ChangeDesignStyle ( $Shop, $StyleTemplateID ) if defined $StyleTemplateID;
    $StyleTemplate->set( { 'IsTemporary' => 0 } );
    $Shop->set( { 'DefaultStyle' => $StyleTemplate } );
    DeleteTemporaryStyles($Shop);

    #resize image to given size if needed
    my $ScaleLogo = $hShopWizardAttributes->{'ScaleLogo'};
    if ($ScaleLogo && defined $hShopWizardAttributes->{'Logo'}) {
        _ResizeLogo($Shop, $hShopWizardAttributes->{'Logo'});
    }

    #save wizard attributes
    SaveWizard($Shop, $hShopWizardAttributes);

    #set user attributes
    _SetUserAttributes($Shop, $Servlet);

    TriggerHook('UI_FinishShopWizard', {
        'Servlet' => $Servlet,
        'Data' => $hShopWizardAttributes,
        'Shop' => $Shop
    });

    ProcessEbizContent($Shop, $hShopWizardAttributes);

    my $hItems = $Shop->get('NextSteps') // {};
    my $CurrentUser = $Servlet->user;
    @$hItems{ qw /Branch Design ShopAddress/ } = (1,1,1);
    $Shop->set({ 'NextSteps' => $hItems });
    $CurrentUser->set({'StartPageTab' => 2});

    return;
}

sub SaveUnityWizardAttributes {
    my $self = shift;
    my ($Servlet) = @_;

    # EPG-10403: there is a special JavaScript monitor for this page
    $Servlet->monitor->blockTimeOut;

    my $Shop = $Servlet->object();
    # merge stored data from session with supplied form data
    my $hShopWizardAttributes = {
        # We need the values utf8-encoded to store them, so do NOT utf8-decode the json string.
        %{JSON->new->utf8(0)->decode( GetSessionAttribute( $Servlet->intSessionID, 'ShopWizardData') )},
        %{$Servlet->form->form($Shop, 'ShopWizardFinish')},
    };

    # save wizard attributes
    eval {
        SaveWizard($Shop, $hShopWizardAttributes);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'UNITY_API_CALL_ERROR') {
            LogError($Error);
            $Servlet->form->executeFormError({
                'Reason'=> 'UNITY_API_CALL_ERROR',
                'Name'  => 'Logo',
                'Form'  => 'ShopWizardContact',
                'GUID'  => $Error->guid
            });
        } else {
            Error($Error);
        }
    }

    # set user attributes
    _SetUserAttributes($Shop, $Servlet);

    TriggerHook('UI_FinishShopWizard', {
        'Servlet' => $Servlet,
        'Data' => $hShopWizardAttributes,
        'Shop' => $Shop
    });

    ProcessEbizContent($Shop, $hShopWizardAttributes);

    # set InitialShopWizardStartup to 0 here because we do not run through the storefront preview
    $Shop->set({'InitialShopWizardStartup' => 0,
                'UseUnityShopLogo' => 1
               });

    return;
}

sub _SetUserAttributes {
    my ($Shop, $Servlet) = @_;

    my $CurrentUser = $Servlet->user;
    my $Address = $Shop->get('Address');

    $CurrentUser->set({'EMail' => $Address->get('EMail')}) unless defined $CurrentUser->get('EMail');
    $CurrentUser->set({'Name' => $Address->get('FullName')});

    # switch locales for user
    $Servlet->locale(1);
}

sub _ResizeLogo {
    my ($Shop, $Logo) = @_;

    my $Style = $Shop->get('DefaultStyle');
    my $File = $Shop->get('PublicPath').'/'.$Logo;
    # force ScaleLogoSize to be integer
    my $ScaleLogoSize = $Style->get('ScaleLogoSize');
    my $OldShopLogo = defined $Shop->get('Logo') ? $Shop->get('PublicPath') . '/' . $Shop->get('Logo') : undef;
    DeleteFile($OldShopLogo) if defined $OldShopLogo and ExistsFile($OldShopLogo) and not $OldShopLogo eq $File;
    if( ExistsFile( $File ) ) {
        if ($ScaleLogoSize < 0) {
            # scale by width
            ScaleImage( $File, $File, $ScaleLogoSize * -1 );
        } else {
            # scale by height
            ScaleImage( $File, $File, $ScaleLogoSize, 1 );
        }
    }
}

sub TestShopWizard {
    my $self = shift;
    my ($Servlet) = @_;

    # get the current object from the servlet
    my $Shop = $Servlet->object();

    # validate input data and get unformatted values
    my $hValues = $Servlet->form->form( $Shop, 'ShopWizard' );
    my $EMail = $hValues->{'EMail'};

    $Servlet->form->executeFormError({
        'Reason'=> 'NO_EMAILADDRESS',
        'Name'  => 'EMail',
    }) unless defined $EMail;

    return;
    # Comment: this function is needed to throw FormErrors
}

sub StartShopWizard {
    my $self = shift;
    my ($Servlet) = @_;

    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardData', '' );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyle', '' );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardSelectedBranch', '' );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyleHeaderImages', '' );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyleBodyImages', '' );

    return;
}

sub SaveShopWizardTopic {
    my $self = shift;
    my ($Servlet) = @_;

    my $SelectedBranchID = $Servlet->params('SelectedBranch');
    my $SelectedBranch   = LoadRootObject()->loadObject($SelectedBranchID, 'StyleGroup');

    my $hSelectedBranch = {
        'ID' => $SelectedBranchID
        , 'Alias' => $SelectedBranch->alias
        , };
    my @hHeaderImages = map {{'ImageFile' => $_->get('ImageFile')}} @{$SelectedBranch->get('StyleHeaderImages')};
    my @hBodyImages = map {{'ImageFile' => $_->get('ImageFile'), 'TileImage' => $_->get('TileImage')}} @{$SelectedBranch->get('StyleBodyImages')};

    SetSessionAttribute( $Servlet->intSessionID, 'SelectedBranchAlias', $SelectedBranch->alias );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardSelectedBranch', to_json($hSelectedBranch) );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyleHeaderImages', to_json(\@hHeaderImages) );
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyleBodyImages', to_json(\@hBodyImages) );

    return;
}

sub SaveShopWizardLayout {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $StyleTemplateID = $Servlet->params('StyleTemplate');
    my $ChangeStyle = $Servlet->params('ChangeStyle');

    TestInteger( 'StyleTemplate', $StyleTemplateID );

    # Change design style.
    my $Style;
    $Style = ChangeDesignStyle ( $Shop, $StyleTemplateID );

    # Set it as temporary to delete it later.
    $Style->set( { 'IsTemporary' => 1 } );

    my $hStyle = {
        'ID'                 => $Style->id,
        'Alias'              => $Style->alias,
        'WizardColor1'       => $Style->get('WizardColor1'),
        'WizardColor2'       => $Style->get('WizardColor2'),
        'WizardColor3'       => $Style->get('WizardColor3'),
        'WizardImage1'       => $Style->get('WizardImage1'),
        'WizardImage2'       => $Style->get('WizardImage2'),
        # Write default colors for wizard to session attribute
        'WizardDefaultColors' => [
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor0') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor0'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor1') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor1'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor2') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor2'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor3') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor3'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor4') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor4'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor5') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor5'}},
        ],
        'TemplateID'         => $StyleTemplateID,
        'TemplateAlias'      => LoadRootObject()->loadObject($StyleTemplateID, 'StyleTemplate')->alias,
        'ChangeStyle'        => $ChangeStyle,
        'UseHeaderBackgroundImage' => $Style->get('UseHeaderBackgroundImage'),
    };

    # EPG-24092: Add eBiz shop dependend navelements to navbar.
    AddEBizElementsToNavBar($Shop, $Style);

    # Store settings in session.
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyle', to_json($hStyle) );

    return;
}

sub SaveShopWizardDesign {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop    = $Servlet->object();
    my $StyleID = $Servlet->params('Style');

    my $Style   = $Shop->loadObject($StyleID, 'Style');

    my $hStyle = {
        'ID'              => $Style->id,
         'Alias'         => $Style->alias,
         'WizardColor1'  => $Servlet->params('WizardColor1'),
         'WizardColor2'  => $Servlet->params('WizardColor2'),
         'WizardColor3'  => $Servlet->params('WizardColor3'),
         'WizardImage1'  => $Servlet->params('WizardImage1'),
         'WizardImage2'  => $Servlet->params('WizardImage2'),
        # Write default colors for wizard to session attribute
        'WizardDefaultColors' => [
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor0') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor0'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor1') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor1'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor2') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor2'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor3') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor3'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor4') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor4'}},
            {'WizardDefaultColor'  => $Style->get('UserDefinedColor5') // $WIZARD_DEFAULT_COLORS->{'WizardDefaultColor5'}},
        ],
         'TemplateID'    => $Servlet->params('StyleTemplate'), 'TemplateAlias' => LoadRootObject()->loadObject($Servlet->params('StyleTemplate'),
         'StyleTemplate')->alias,
         'ChangeStyle'   => 0,
         'UseHeaderBackgroundImage' => $Style->get('UseHeaderBackgroundImage'),
         };

    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardStyle', to_json($hStyle) );
    return;
}

sub SaveShopWizardContact {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $ShopWizardData = GetSessionAttribute($Servlet->intSessionID, 'ShopWizardData');

    my $hShopWizardAttributes;
    if (defined $ShopWizardData && $ShopWizardData ne '') {
        $hShopWizardAttributes = {(%{from_json($ShopWizardData)},
            %{$Servlet->form->form($Shop, 'ShopWizardContact')})};
    }
    else {
        $hShopWizardAttributes = $Servlet->form->form($Shop, 'ShopWizardContact');
    }

    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardData', to_json($hShopWizardAttributes) );

    return;
}

sub SaveUnityShopWizardContact {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form    = $Servlet->form;

    # shop name is mandatory for Unity
    my $Name = $Form->value('Name');
    $Form->executeFormError({
       'Form'  => 'ShopWizardContact',
       'Reason'=> 'FORM_NO_NAME',
       'Name'  => 'Name'
    }) unless $Name;

    $self->SaveShopWizardContact($Servlet);
}

sub SaveShopWizardSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    # ordering is relevant because older values from session should be overwritten by newer ones
    my $hShopWizardAttributes = {(
        %{from_json(GetSessionAttribute($Servlet->intSessionID, 'ShopWizardData'))},
        %{$Servlet->form->form($Shop, 'ShopWizard')},
        %{$Servlet->form->form($Shop, 'ShopWizardShopSettings')}
    )};
    SetSessionAttribute( $Servlet->intSessionID, 'ShopWizardData', to_json($hShopWizardAttributes) );

    return;
}
#==============================================================================
# Shows the Storefront after first time clicking throw setup wizard.
#
# See AD-805 for more details.
#==============================================================================
sub ViewStorefrontPreview {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    # if shop wizard was already shown: Go to MBO.
    unless ($self->isPreviewShown($Shop)) {
        my $Location = $Servlet->baseLineContentLocation . "?ViewAction=MBO-ViewGeneral&ObjectID=" . $Shop->get('ID');
        $Servlet->redirect($Location);
    }

    # never show the view Storefront preview a second time.
    $Shop->set({'InitialShopWizardStartup' => 0});

    return;
}

sub isPreviewShown {
  my $self = shift;
  my ($Shop) = @_;

  my $hInfo = {
      'hookCondition' => 1,
      'Shop'          => $Shop,
  };

  TriggerHook('UI_IsPreviewShown', $hInfo);

  return $Shop->get('InitialShopWizardStartup') && $hInfo->{'hookCondition'};
}

1;
