#========================================================================================
# package      DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedlyPayULatam
# state        public
# base         DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly
#----------------------------------------------------------------------------------------
# description  object interface for Spreedly-PayU Latam Payment.
#========================================================================================
package DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedlyPayULatam;
use base DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly;
use base DE_EPAGES::Payment::UI::PaymentMethod;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Spreedly::API::Constants qw ( %PAYULATAM_DEFAULTS );

use constant FEATURE_NAME => 'SpreedlyPayULatam';

use strict;

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP, USD
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    my $AreCurrenciesSupported = 1; # PayULatam supports almost every currency
    my $CanBeAddedToBasket = $AreCurrenciesSupported && $self->_callSuperCanAddToBasket($hVars);

    return $CanBeAddedToBasket;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedlyPayULatam->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>.
#               It checks if request comes from NewSpreedlyMethod, if not, we change name.
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    my $DefaultPaymentTypeMethod = $class->_getDefaultPaymentTypeMethod();

    delete($hInfo->{'Name'});

    my $BaseName = $DefaultPaymentTypeMethod->alias();
    if (not isValid($hInfo->{'PaymentTypeMethod'})) {
        $hInfo->{'Alias'} = $class->createNewPaymentMethodName($Parent, $BaseName);
    }

    return {
        %{$class->_callSuperDefault($Parent, $hInfo)},
        'PaymentTypeMethod' => $DefaultPaymentTypeMethod
    };
}

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description  Returns feature name.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return FEATURE_NAME; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod = DE_EPAGES::Payment::API::Object::PaymentMethod->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethod');
#               my $PaymentMethod = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  first SUPER::instert is called, then we set the default payment method
#               name for all languages installed to the shop.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentMethod | new object | object
# hook         OBJ_InsertPaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->_callSuperInsert($hInfo);
    my $Parent = $self->parent();
    my $Shop = $Parent->getSite();
    my $Name = $self->get('Name', $Shop->get('LanguageID'));

    if (not isValid($Name)) {
        my $DefaultPaymentTypeMethod = $class->_getDefaultPaymentTypeMethod() ;
        $self->_setDefaultNameForAllLanguages($DefaultPaymentTypeMethod, $Shop);
    }

    return $self;
}

sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $bCredentialsAreDefined = defined $self->get( 'ApiLogin' )
        && defined $self->get( 'ApiKey' )
        && defined $self->get( 'MerchantID' )
        && defined $self->get( 'AccountID' )
        && defined $self->get( 'PaymentCountry' );

    return  $bCredentialsAreDefined && $self->_callSuperUsableAtStorefront( $hVars );
}
#----------------------------------------------------------------------------------------
# private
#----------------------------------------------------------------------------------------

sub _setDefaultNameForAllLanguages {
    my $self = shift;
    my ($DefaultPaymentTypeMethod, $Shop) = @_;

    my $hInstalledLanguages = $Shop->get('Languages');

    foreach my $Language (@{ $hInstalledLanguages }) {
        my $LanguageID = $Language->{'LanguageID'};
        my $BaseName = $DefaultPaymentTypeMethod->get('NameOrAlias', $LanguageID);

        $self->set({'Name' => $BaseName}, $LanguageID);
    }
}



sub _getDefaultPaymentTypeMethod {
    my $self = shift;

    return LoadObjectByPath($PAYULATAM_DEFAULTS{'PaymentTypeMethod_Object_Path'});
}

sub _callSuperDefault {
    my $self = shift;
    my ($Parent,$hInfo) = @_;

    return $self->SUPER::default($Parent, $hInfo);
}

sub _callSuperCanAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return $self->SUPER::canAddToBasket($hVars);
}

sub _callSuperInsert {
    my $self = shift;
    my ($hInfo) = @_;

    return $self->SUPER::insert($hInfo);
}

sub _callSuperUsableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    return $self->SUPER::usableAtStorefront($hVars);
}
1;